<?php
/*
	Exports diary.ru blog contents to RSS.
	Usage:
		diary_export.php?login=himself&from=20
	Required:
		login   = diary.ru login
	Optional
		from    = starting post (default = 1 = latest)
		to		= ending post (default = 0 = one page)
		tag		= 2113 (tag index, default = 0 = no filter)
		more	=
			html:		<a class=1>stub</a><span class=2>content</span>
			shortcode:	[more=stub]content[/more]
		barehtml	=
			false:	&lt b &gt		(as required in RSS)
			true:		<b>				(for debug, human-readable)
		encoding	=
			utf-8
			windows-1251
	Warning.
		This file should be kept in UTF-8 encoding.
*/
	header("Cache-Control: no-cache");
	require 'portable.php';

	$s_login = $_GET['login'];
	if ($s_login=='')
		die("Login required.");
	$s_from = $_GET['from'];
	$s_to = $_GET['to'];
	$bare_html = ($_GET['barehtml']=="true");
	
	/* Supported encodings: UTF-8, Windows-1251
	  If we are to support more, we'll have to test for validity */
	$encoding = strtolower($_GET['encoding']);
	if ($encoding=='utf-8')
		$encoding='';
	else
	if ($encoding<>'windows-1251')
		$encoding='';

	/* All processing is in UTF-8. Only output differs */
	if ($encoding)
		header('Content-Type: text/html; charset='.$encoding);
	else
		header('Content-Type: text/html; charset=utf-8');
	mb_internal_encoding('UTF-8'); /* wtf PHP it's 2011 already */
	mb_regex_encoding('UTF-8');

	/* Defaults */
	if ($s_from=="") $s_from="1";
	if ($s_to=="") $s_to="0";

	/* Sanitize inputs */
	$login = preg_replace("/[^a-zA-Z0-9\s]/", "", $s_login);
	$from = (int)$s_from;
	$to = (int)$s_to;
	$tag = (int)$_GET['tag'];
	if ($from<0) $from = 1;
	if ($to<0) $to = 0;
	if ($tag<0) $tag = 0;

	include("curlext.php");
	include("domext.php");

	/* Don't optimize $diaryru, $addr away. Need them later. */
	$diaryru = "http://www.diary.ru";	/* Server */
	$addr = $diaryru."/~".$login; 		/* Blog address */

	/* Fetches page and parses it's DOM. Returns HTTP error code. */
	$pages_fetched = 0;
	function fetch_dom($addr, &$dom) {
		global $pages_fetched;
		echo "<!-- Fetching page: ".$addr." -->\n";
		$ret = curl_fetch($addr);
		if ($ret[0]!=200) {
			$dom = null;
			return $ret[0];
		}
		$pages_fetched++;
		//$ret[1] = enc_html_entities($ret[1]);
		//iconv("windows-1251", "utf-8", )
		$dom = new DOMParser();
		$dom->test($ret[1]);
		return $ret[0];
	}

	function enc_html_entities($s) {
		return preg_replace_callback("/<(pre|nowiki)>(.*)<\/\1>/i", "enc_html_callback", $s);
	}
	function dec_html_entities($s) {
		return preg_replace_callback("/<(pre|nowiki)>(.*)<\/\1>/i", "dec_html_callback", $s);
	}
	function enc_html_callback($match) {
		echo "match";
		return "<".$match[1].">".preg_replace(
			array("/&(#\d*;?)/i", "/([^#])(##*\d*;?)/iu"),
			array("#\1", "\1#\2"),
			$match[2])."</".$match[1].">";
	}
	function dec_html_callback($match) {
		return "<".$match[1].">".preg_replace(
			array("/([^#])#(#\d*;?)/i", "/([^#])#(#+\d*;?)/iu"),
			array("\1&\2", "\1\2"),
			$match[2])."</".$match[1].">";
	}

	/* Active index page */
	$paddr = NULL;
	$DOM = NULL;

	/* Fetches index page. Returns false if not available, dies on unknown errors */
	function fetch_index($from) {
		global $addr, $paddr, $DOM, $tag;
		$paddr = $addr."/?from=".$from;		/* Index page address */
		if ($tag > 0)
			$paddr = $paddr."&tag=".$tag;
		$rs = fetch_dom($paddr, $DOM);
		if (($rs!=200) && ($rs!=404))
			die('Cannot fetch index page: '.$paddr.'.<br \>Error: '.$rcode.'.');
		return ($rs==200);
	}

	/* Diary only gives out from=20, 40, etc... */
	$page_sz = 20;
	$page_first = (($from-1)-($from-1)%$page_sz)/$page_sz;
	if ($to<=0) $to=$from+$page_sz-1;
	if ($to>$from)
		$page_last = (($to-1)-($to-1)%$page_sz)/$page_sz;
	else
		$page_last = $page_first;

	/* Current page and post */
	$cur_page = $page_first;
	$cur_post = $page_first * $page_sz;

	/* Fixes character encoding -- applied to all output */
	function fixenc($text) {
		global $encoding;
		if (!$encoding) return $text;
		return iconv("UTF-8", $encoding, $text);
	}

	/* Encodes special chars */
	function htmlenc($text) {
		global $bare_html;
		if (!$bare_html)
			return htmlspecialchars(dec_html_entities($text));
		else
			return dec_html_entities($text);
	}

	/* Must be first line in a file */
	if ($encoding)
		echo '<?xml version="1.0" encoding="'.$encoding.'"?>';
	else
		echo '<?xml version="1.0" encoding="utf-8"?>';
?>

<rss version="2.0" xmlns:content="http://purl.org/rss/1.0/modules/content/" xmlns:diary="http://diary.ru/">

<?php
	echo "<!-- Diary export running. -->\n";
	echo "<!-- Target pages: from ".$page_first." to ".$page_last." -->\n";

	/* Fetch first page because we need some generic information (blog title). */
	if (!fetch_index($cur_post)) {
		/* Если нет даже первого поста, читаем главную страницу.
		 Впрочем, сейчас дайри всегда возвращает _предпоследнюю_ страницу. */
		if (!fetch_index(0))
			die('Blog not found.');
		/* Все посты пропускаем */
		$cur_page = $page_last+1;
		$cur_post = $cur_page * $page_sz+1;
	}
?>

<channel>
	<title><?php
		$nodeTitle=$DOM->getElementsByTagName('title')->item(0);
		$title=htmlenc(fixenc($nodeTitle->textContent));
		echo $title;
	?></title>
	<link><?php echo $addr; ?></link>
	<generator>DiaryExport</generator>
	<ttl>60</ttl>

<?php
	/* Заменяем все символы в дате на международные, для удобства парсинга */
	$fix_date_pattern = array(
		'/понедельник/iu', '/вторник/iu', '/среда/iu', '/четверг/iu', '/пятница/iu', '/суббота/iu', '/воскресенье/iu',
		'/января/iu', '/февраля/iu', '/марта/iu', '/апреля/iu', '/мая/iu', '/июня/iu', '/июля/iu',
		'/августа/iu', '/сентября/iu', '/октября/iu', '/ноября/iu', '/декабря/iu');
	$fix_date_replace = array(
		'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday',
		'January', 'February', 'March', 'April', 'May', 'June', 'July',
		'August', 'September', 'October', 'November', 'December');
	function fix_date($date) {
		global $fix_date_pattern;
		global $fix_date_replace;
		$tmp = preg_replace($fix_date_pattern, $fix_date_replace, $date);
		return $tmp;
	}
	
	$activeDate = 0; /* Last found Date mark */
	/* Parses postDate node and updates activeDate */
	function parseDate($postDate) {
		global $DOM;
		$span = $DOM->ngetType('span', $postDate);
		if (!$span) return;

		$dateText = $span->textContent;
		echo "<!-- postDate: ".fixenc($dateText)."-->\n";

		/* Вначале заменяем слова на английские */
		$dateText = fix_date($dateText);

		echo '<!-- fixedDate: '.fixenc($dateText)." -->\n";

		/* Теперь декодируем */
		$date = strptime($dateText, "%A, %d %B %Y");
		
		global $activeDate;
		$activeDate = $date;
	}

	$postDOM = NULL; /* When loaded, contains post DOM */
	$permaPostContent = NULL;

	/* Loads a permanent post version and establishes post content */
	function need_post_DOM($addr) {
		global $postDOM;
		if ($postDOM!=NULL) return;
		$rs = fetch_dom($addr, $postDOM);
		if ($rs!=200)
			die('Cannot fetch post: '.$addr.'.<br />Error code: '.$rs.'.');

		global $permaPostContent;
		/* We only have one singlePost, so skip that */
		$permaPostContent = $postDOM->ngetClass('postContent');
		$permaPostContent = $postDOM->ngetClass('paragraph', $permaPostContent);
		$permaPostContent = $postDOM->ngetType('div', $permaPostContent);
	}

	function drop_post_DOM() {
		global $postDOM, $permaPostContent;
		$postDOM = NULL;
		$permaPostContent = NULL;
	}


	/*
		When hidden, more tag looks like this:
			<b>More</b>: <a href=#more1>stub text</a>
		Open, it becomes:
			<b>More</b>: <a href=#more1 /> Content <a href=#more1end />
		Therefore to fetch content we look for IN and OUT markers.

		But there's a trick. It's possible to do dirty things:
			[more=<b>stub</b>] text [/more]									- proper HTML DOM in stub
			<b>[more=stub]</b> text <b>[/more]</b>					- tricky DOM
			<b>[more=stub</b>]replacement</b> text[/more]		- fucked up DOM
		Diary permits the first two, so we have to account for those.

		DOM trees for second type:
			[parent] - <b> - <a href> - stub (bold)
			[parent] - <b> - <IN>
						\- contents (not bold)
						\- <b> - <OUT>
	*/

	/* Patches [more] contents. Shortcode version */
	function patch_more_shortcode($moreid, $moreLink) {
		/*
			1. Find <IN>.
			2. Go up until one level below common parent with <OUT>.
			3. Replace <IN> with [more=(...)]
			4. Move this node replacing <a href>
			5. Move all siblings after <a href>, until the one containing <OUT>
			6. Replace <OUT> with [/more]
		*/
		/*
			Сейчас это полумеры: нужен аккуратный алгоритм:
			- Один общий предок может содержать несколько more. Поэтому нужно не копировать его содержимое из поста,
				а дополнять содержимое в стаб-посте по мере прохождения.
			- То, что было в стаб-посте справа от ссылки, необходимо сохранить перед прохождением,
			  и вставить после всех вставленных content справа от OUT.
			  При этом надо проверить, что мы на нужном уровне и с нужной цепью элементов (иначе нужно подняться наверх
			  и воссоздать правильную).
		*/
		global $DOM, $postDOM, $permaPostContent;

		/* Find opening and ending anchors */
		$in = $postDOM->ngetName($moreid, $permaPostContent);
		$out = $postDOM->ngetName($moreid."end", $permaPostContent);
		if (!$in && !$out) die("Invalid more structure: no opening or closing anchor.");

		/* Find common parent, select first relevant children */
		$top = $in;				/* first relevant children */
		$ptr = $moreLink; /* it's counterpart at short version of the post */
		while ($top->parentNode && !DOMisParent($top->parentNode, $out)) {
			$top = $top->parentNode;
			$ptr = $ptr->parentNode;
		}
		if (!$top->parentNode || !$ptr) die("Invalid [more] structure: cannot establish opening and closing anchor's common parent.");

		/* Replace <IN> with "[more=(stub)]". Note that dom for stub MUST be consistent. */
		$new_in = $postDOM->createTextNode("[more=");
		$in->parentNode->replaceChild($new_in, $in);
		if ($top===$in) /* could have been on the same level */
			$top=$new_in; /* note that we need the leftmost of new nodes [more, stub, ] */

		/* Rest of stub */
		$new_in=DOMimportChildrenAfter($new_in, $moreLink); /* Inserting as DOM and not text because stub can contain HTML. */
		$tmp=$postDOM->createTextNode("]");
		DOMinsertAfter($new_in, $tmp);
		$new_in=$tmp;

		/* Replace <a href> with <this> at (1) */
		$ptr = DOMreplace($ptr, $DOM->importNode($top, true));

		/* Scan siblings */
		while ($top->nextSibling && !DOMisParent($top->nextSibling, $out)) {
			$top = $top->nextSibling;
			$ptr = DOMinsertAfter($ptr, $DOM->importNode($top, true));
		}
		if (!$top->nextSibling) die("Invalid [more] structure: cannot establish closing anchor's entry parent.");
		$top = $top->nextSibling;

		/* Last sibling */
		$new_out = $postDOM->createTextNode("[/more]");
		$out->parentNode->replaceChild($new_out, $out);
		if ($top===$out) /* could have been on the same level */
			$top=$new_out;
		$ptr = DOMinsertAfter($ptr, $DOM->importNode($top, true));
	}

	/* Patches [more] contents. Plain html version */
	function patch_more_html($moreid, $moreLink) {
		/*
		  We start at IN and walk forward copying siblings until we find the one
		  which contains <OUT>.
		  If there are none, we go one level up and continue with the same:
		    - <IN>			(no siblings, up)
		    - <b> - <IN>	(next sibling)
		    \- contents     (next sibling)
		    \- <b>			(contains <OUT>, enter)
		      - <b> - <OUT> (<OUT>)

		  This replicated subtree is attached at [parent] (1),
		  after a node which ultimately contains <a href=#more1>.

		  This way, all external tags are closed, and we reopen them inside of more.
		*/
		global $DOM, $postDOM, $permaPostContent;

		/* Find opening anchor */
		$moreOp = $postDOM->ngetName($moreid, $permaPostContent);

		/* Create cloning target */
		$ptr		= $moreLink;						/* where to insert the tree */
		$targettop	= $DOM->createElement("span");		/* topmost cloned parent */

		$target	= $targettop;	/* where we place cloned nodes at this level */
		$source = NULL;			/* sibling level we scan */
		if ($moreOp->nextSibling)
			$source=$moreOp->nextSibling;
		else
			$source=$moreOp->parentNode;
		if (!$source) die('Invalid [more] structure: no parent at <IN> point.');

		$stop = false;
		while (!$stop) {
			/* <OUT> */
			if (($source instanceof DOMElement)
			&& ($source->tagName=="a")
			&& ($source->getAttribute("name")==$moreid."end")) {
				$stop = true;
				break;
			}

			/* clone the node */
			$clone = $DOM->importNode($source, false)
				or die('Cannot import node '.$source->nodeName);
			$target->appendChild($clone);

			/* choose next action */
			if ($source->firstChild) {
				/* down */
				$source = $source->firstChild;
				$target = $clone;
			} else
			if ($source->nextSibling) {
				/* forward */
				$source = $source->nextSibling;
			} else {
				/* Ищем ближайшего незакрытого предка */
				while ($source && $target && !$source->nextSibling) {
					/* up */
					if ($target==$targettop) {
						/* replace top with clone */
						$clone = DOMimportSingleNode($DOM, $source->parentNode);
						DOMmoveChildren($targettop, $clone);
						/* create new top, insert clone */
						$targettop = $DOM->createElement("span");
						$targettop->appendChild($clone);
						$target = $targettop;
						/* ptr one level up */
						$ptr = $ptr->parentNode;
					} else {
						$target = $target->parentNode;
					}
					$source = $source->parentNode;
				}

				/* found node with sibling */
				if ($source) { /* иначе потом разберём */
					$source=$source->nextSibling;
				}
			} /* end of choosing next action */

			if (!$source)
				die('Invalid [more] structure: no more-end anchor found.');
			if (!$target)
				die('Cannot parse [more] structure. Probably my fault so tell the developers.');
		}

		/* We now have cloned tree in $targettop. Let's insert it after $ptr */
		$targettop->setAttribute("class", "ContentsMore");
		$targettop->setAttribute("id", $moreid."-contents");
		DOMinsertAfter($ptr, $targettop);
	}



	/* For every page */
	for (; $cur_page<=$page_last; $cur_page++) {
		drop_post_DOM();
		echo "<!-- Page from=".$cur_post."-->\n";

		/* First page is already loaded */
		if ($cur_page > $page_first)
			if (!fetch_index($cur_post))
				break; /* no more posts */

		/* For every post */
		$items=$DOM->ngetClassM('singlePost');
		echo "<!-- count: ".count($items)."-->\n";
		foreach ($items as $item) {
			echo "<!-- Post id: ".$item->getAttribute("id")."-->\n";

			/* Skip epigraph - it lacks many parts of a post */
			$postid = $item->getAttribute("id");
			if (($postid=="epigraph") || ($postid=="post0"))
				continue;

			/* Update post counter */
			$cur_post++;
			if ($cur_post < $from)
				continue;
			if (($to >= $from)&&($cur_post > $to))
				break;

			/* Clear post data */
			drop_post_DOM();

			/* If there's a postDate, parse it */
			$postDate = $DOM->ngetClassM('postDate', $item);
			if ($postDate && ($postDate->length>0))
				parseDate($postDate->item(0));

			/* Start new item */
			echo "<item>\n";
			$postTitle = $DOM->ngetClass('postTitle', $item);
			
			/* Time */
			$postTime = $DOM->ngetType('span', $postTitle);
			$timeText = $postTime->textContent;
			echo "<!-- postTime: ".$timeText."-->\n";
			$timeParts = strptime($timeText, "%H:%M");
			$time = mktime(
				$activeDate['tm_hour'] + $timeParts['tm_hour'],
				$activeDate['tm_min'] + $timeParts['tm_min'],
				$activeDate['tm_sec'],
	            $activeDate['tm_mon'] + 1,
	            $activeDate['tm_mday'],
	            $activeDate['tm_year'] + 1900);
			/* will be printed later */

			/* Title */
			$nodeTitle = $DOM->ngetType('h2', $postTitle);
			$title=htmlenc(fixenc($nodeTitle->textContent));
			echo "<title>".$title."</title>\n";

			/* Permalink */
			$nodeLink = $DOM->ngetClass('postLinksBackg', $item);
			$nodeLink = $DOM->ngetClass('urlLink', $nodeLink);
			$nodeLink = $DOM->ngetType('a', $nodeLink);
			$permalink=$diaryru.$nodeLink->getAttribute('href');
			echo "<guid isPermalink=\"true\">".$permalink."</guid>\n";
			echo "<link>".$permalink."</link>\n";
			echo "<comments>".$permalink."</comments>\n";

			/* Content */
			$contentLink = $DOM->ngetClass('postContent', $item);
			$contentLink = $DOM->ngetClass('paragraph', $item);
			$dataLink = $DOM->ngetType('div', $contentLink);

			/* [more] links */
			/* DOMNodeList auto-deletes removed nodes, so copy */
			$tmp = $DOM->ngetClassM('LinkMore', $dataLink);
			$moreLinks=array(); /* reset */
			for ($i=0; $i<$tmp->length; $i++)
				$moreLinks[$i]=$tmp->item($i);
			foreach ($moreLinks as $moreLink) {
				/* Decode moreid */
				if ((preg_match('/\#(.*)/', $moreLink->getAttribute('href'), $match)<1) || !isset($match[1]))
					continue;
				$moreid = $match[1];
				echo "<!-- moreid: ".$moreid." -->\n";

				/* Fetch permaPostContent if not yet */
				if (strpos($permalink, '?')===FALSE)
					$permalink_oam=$permalink.'?oam';
				else
					$permalink_oam=$permalink.'&oam';
				need_post_DOM($permalink_oam); /* more теперь по умолчанию закрыты */

				/* Patch */
				if ($_GET['more']==shortcode)
					patch_more_shortcode($moreid, $moreLink);
				else
					patch_more_html($moreid, $moreLink);
			}

			/* Post content */
			$postData = htmlenc(fixenc(DOMinnerXml($DOM, $dataLink)));
			echo "<description>".$postData."</description>\n";

			/* Categories */
			$categoryLink = $DOM->ngetClassN('atTag', $contentLink);
			if ($categoryLink)
				foreach ($DOM->ngetTypeM('a', $categoryLink) as $category) {
					$catname = fixenc($category->textContent);
					echo "<category>".$catname."</category>\n";
				}

			echo "<pubDate>".gmdate(DATE_RFC2822, $time)."</pubDate>\n";
			echo "<author>".$login."</author>\n";
			echo "</item>\n\n";
		}
	}

	/* Output stats */
	echo "<!-- Pages fetched: ".$pages_fetched." -->\n";
?>
</channel>
</rss>