<?php
/* DOM Utils */

	function DOMinnerHTML($dom, $element)
	{
	    $innerHTML = "";
	    $children = $element->childNodes;
	    foreach ($children as $child)
	    {
	        $tmp_dom = new DOMDocument();
	        $tmp_dom->appendChild($tmp_dom->importNode($child, true));
	        $innerHTML.=$tmp_dom->saveHTML();
	    }
	    return $innerHTML;
	}

	/* innerXML does not garble barble non-latin text into &$FAED codes */
	function DOMinnerXML($dom, $element)
	{
	    $innerXML = "";
	    $children = $element->childNodes;
	    foreach ($children as $child)
	    	$innerXML.=$dom->saveXML($child);
	    return $innerXML;
	}

	/* Imports DOM node including attributes but not children elements */
	function DOMimportSingleNode($dom, $node) {
		$cnode = $dom->importNode($node, false);
		foreach ($node->childNodes as $child) {
			$tp = $child->nodeType;
			if ($tp==XML_ATTRIBUTE_NODE) /*    */
				$cnode->appendChild($dom->importNode($child, false));
		}
		return $cnode;
	}

	/* Moves all node children to another node. Nodes must be in the same DOM */
	function DOMmoveChildren($source, $target) {
		while ($source->firstChild) {
			$child = $source->firstChild;
			$source->removeChild($child);
			$target->appendChild($child);
		}
	}

	/* Replaces one node with another, returns a new node */
	function DOMreplace($ptr, $node) {
		$ptr->parentNode->replaceChild($node, $ptr);
		return $node;
	}

	/* Inserts $node immediately after $place */
	function DOMinsertAfter($place, $node) {
		$nextNode = $place->nextSibling;
		if (!$nextNode) //no next sibling
			return $place->parentNode->appendChild($node);
		else
			return $place->parentNode->insertBefore($node, $nextNode);
	}

	/* Imports $node and inserts immediately after $place.
	 Returns imported node. */
	function DOMimportAfter($place, $node) {
		$inode=$place->ownerDocument->importNode($node, true);
		DOMinsertAfter($place, $inode);
		return $inode;
	}

	/* Imports $node children and inserts those immediately after $place.
	 Returns last imported node. */
	function DOMimportChildrenAfter($place, $node) {
		$child = $node->firstChild;
		while ($child) {
			$place = DOMimportAfter($place, $child);
			$child = $child->nextSibling;
		}
		return $place;
	}

	/* Returns true if $parent contains $node */
	function DOMisParent($parent, $node) {
		if (!$parent || !$node) return false;
		while ($node && ($node!==$parent))
			$node=$node->parentNode;
		return ($node!=null);
	}

	/* Walks the DOM tree.
	  $item_callback($data, $item) can return false to stop and return this node.
	  $move_callback($data, $direction, $new_item)
	  $directions:
	    down, next, up, upnext
	*/
	function DOMwalk($start, $item_callback, $move_callback, $data) {
		$node = $start;
		while (!$stop) {
			if (is_callable($item_callback)) {
				$stop = call_user_func($item_callback, $data, $node);
				if ($stop) break;
			}

			/* choose next action */
			if ($start->firstChild) {
				/* down */
				$node = $node->firstChild;
				if (is_callable($move_callback)
					&& call_user_func($move_callback, $data, "down", $node))
					return false;
			} else
			if ($source->nextSibling) {
				/* forward */
				$node = $node->nextSibling;
				if (is_callable($move_callback)
					&& call_user_func($move_callback, $data, "next", $node))
					return false;
			} else {
				/*     */
				while ($node && !$node->nextSibling) {
					/* up */
					$node = $node->parentNode;
					if (is_callable($move_callback)
						&& call_user_func($move_callback, $data, "up", $node))
						return false;
				}

				if (!$source) /* no more nodes */
					return false;

				$node=$node->nextSibling;
				if (is_callable($move_callback)
					&& call_user_func($move_callback, $data, "upnext", $node))
					return false;
			} /* end of choosing next action */

			if (!$node || !$stop)
				return false;
			return $node;
		}
	}

class DOMParser extends DOMDocument {
	public $xpath;

/*	function __construct() {
		parent::__construct();
		@$this->loadHTML($data);
		$this->xpath = new DOMXPath($this);
	}*/

	function test($data) {
		@$this->loadHTML($data);
		$this->xpath = new DOMXPath($this);
	}

	/*
		Supported NGET expressions:
			ngetSomething		gets first instance of Something or dies('No instance')
			ngetSomethingM		gets all instances of Something (maybe none)
			ngetSomethingN		gets first instance of Something or null
	*/

	/* Returns nodes by type (<a>, <h1>, <span>) */
	function ngetTypeM($nodeType, $parentNode=null) {
		if ($parentNode==null)
			$parentNode = $this;
		return $this->xpath->evaluate(".//".$nodeType, $parentNode);
	}

	function ngetTypeN($nodeType, $parentNode=null) {
		$nodes = $this->ngetTypeM($nodeType, $parentNode);
		if ((!$nodes) || ($nodes->length<=0))
			return null;
		else
			return $nodes->item(0);
	}

	function ngetType($nodeType, $parentNode=null) {
		$nodes = $this->ngetTypeM($nodeType, $parentNode);
		if ((!$nodes) || ($nodes->length<=0))
			die('Node not found: type='.$nodeType);
		return $nodes->item(0);
	}

	/* Returns nodes by name */
	function ngetNameM($nodeName, $parentNode=null) {
		if ($parentNode==null)
			$parentNode = $this;
		return $this->xpath->evaluate(".//*[@name='".$nodeName."']", $parentNode);
	}

	function ngetNameN($nodeName, $parentNode=null) {
		$nodes = $this->ngetNameM($nodeName, $parentNode);
		if ((!$nodes) || ($nodes->length<=0))
			return null;
		else
			return $nodes->item(0);
	}

	/* also checks $parentNode itself */
	function ngetNameN2($nodeName, $parentNode=null) {
		$ret = $this->ngetNameN($nodeName, $parentNode);
		if ($ret) return $ret;
		if (!$ret && $parentNode && ($parentNode instanceof DOMElement)
				&&($parentNode->getAttribute('name')==$nodeName))
			return $parentNode;
	}

	function ngetName($nodeName, $parentNode=null) {
		$nodes = $this->ngetNameM($nodeName, $parentNode);
		if ((!$nodes) || ($nodes->length<=0))
			die('Node not found: Name='.$nodeName);
		return $nodes->item(0);
	}

	/* Returns nodes by CSS class */
	function ngetClassM($cssClass, $parentNode=null) {
		if ($parentNode==null)
			$parentNode = $this;
		return $this->xpath->evaluate(".//*[contains(@class,'".$cssClass."')]", $parentNode);
	}

	function ngetClassN($cssClass, $parentNode=null) {
		$nodes = $this->ngetClassM($cssClass, $parentNode);
		if ((!$nodes) || ($nodes->length<=0))
			return null;
		else
			return $nodes->item(0);
	}

	function ngetClass($cssClass, $parentNode=null) {
		$nodes = $this->ngetClassM($cssClass, $parentNode);
		if ((!$nodes) || ($nodes->length<=0))
			die('Node not found: class='.$cssClass);
		return $nodes->item(0);
	}
}
?>